/*--------------------------------------------------------------------------------*
 *                                                                                *
 * CodecPro Incorporated                                                          *
 * 2162 Av Laurier E                                                              *
 * Montreal (Quebec)                                                              *
 * Canada, H2H 1C2                                                                *
 *                                                                                *
 * www.codecpro.com                                                               *
 *                                                                                *
 * This material is subject to continuous developments and improvements. All      *
 * warranties implied or expressed, including but not limited to, implied         *
 * warranties of merchantability, fitness for purpose, condition of title,        *
 * noninfringement, are excluded. In no event shall CodecPro Incorporated and its *
 * suppliers be liable for any special, indirect or consequential damages or any  *
 * damages whatsoever arising out of or in connection with the use of this        *
 * information. The foregoing disclaimer shall apply to the maximum extent        *
 * permitted by applicable law, even if any remedy fails its essential purpose.   *
 *                                                                                *
 *                     NON-COMMERCIAL use only.                                   *
 *                                                                                *
 * For a commercial version please contact CodecPro at sales@codecpro.com         *
 *                                                                                *
 *--------------------------------------------------------------------------------*/

/*
 *===================================================================
 *  3GPP AMR Floating-point Speech Codec
 *===================================================================
 */

#define _CRT_SECURE_NO_WARNINGS

#include <stdio.h>
#include "typedef.h"
#include "cp_amr.h"

/*
 * Define AMRNB_MAGIC_NUMBER if the magic number "#!AMR\n" is expected to be present 
 * at the beginning of the encoded input file
 */
//#define AMR_MAGIC_NUMBER "#!AMR\n"
#ifdef AMRNB_MAGIC_NUMBER
#include <string.h>
#endif

const UWord8 amrnb_frame_size[16]= {13, 14, 16, 18, 20, 21, 27, 32, 6, 0, 0, 0, 0, 0, 0, 1};

/*
 * DECODER.C
 *
 * Main program of the AMR NB ACELP decoder.
 *
 *    Usage : decoder bitstream_file synth_file
 *
 *    Format for bitstream_file:
 *        Described in TS26.201
 *
 *    Format for synth_file:
 *      Synthesis is written to a binary file of 16 bits data.
 *
 */

int main(int argc, char *argv[])
{
   FILE *f_serial;                        /* File of serial bits for transmission  */
   FILE *f_synth;                         /* File of speech data                   */

   Word16 synth[L_FRAME8k];               /* Buffer for speech @ 16kHz             */
   UWord8 serial[NB_SERIAL_MAX];
   Word16 mode;
   Word32 bfi;
   Word32 frame;

   #ifdef AMRNB_MAGIC_NUMBER
   char magic[16];
   #endif

   printf("\n");
   printf("**************       CodecPro Incorporated        ************\n");
   printf("\n");
   printf("------------------ AMR Floating point decoder ----------------\n");
   printf("\n");
   printf("----------------   (Non-Commercial use only)   ---------------\n");
   printf("\n");

   /*
    * Read passed arguments and open in/out files
    */
   if (argc != 3)
   {
      fprintf(stderr, "Usage : %s  bitstream_file  synth_file\n", argv[0]);
      fprintf(stderr, "\n");
      fprintf(stderr, "Format for bitstream_file:\n");
      fprintf(stderr, "  Described in RFC 4867 (Sections 5.1 and 5.3).\n");
      fprintf(stderr, "\n");
      fprintf(stderr, "Format for synth_file:\n");
      fprintf(stderr, "  Synthesis is written to a binary file of 16 bits data.\n");
      fprintf(stderr, "\n");
      return(0);
   }

   /*
    * Open file for synthesis and packed serial stream
    * and validate input filenames
    */
   if ((f_serial = fopen(argv[1], "rb")) == NULL)
   {
      fprintf(stderr, "Input file '%s' does not exist !!\n", argv[1]);
      return(0);
   }
   if ((f_synth = fopen(argv[2], "wb")) == NULL)
   {
      fprintf(stderr, "Cannot open file '%s' !!\n", argv[2]);
      return(0);
   }

   fprintf(stderr, " Input bitstream file:  %s\n", argv[1]);
   fprintf(stderr, "Synthesis speech file:  %s\n", argv[2]);

   /*
   * Initialization of decoder
   */
   if (D_IF_amrnb_init() != 0)
   {
      fprintf(stderr, "Decoder initialization error!!\n");
      return(0);
   }

   #ifdef AMRNB_MAGIC_NUMBER
   /*
    * read and verify magic number
    */
   fread(magic, sizeof(char), strlen(AMRNB_MAGIC_NUMBER), f_serial);
   if (strncmp(magic, AMRNB_MAGIC_NUMBER, strlen(AMRNB_MAGIC_NUMBER)))
   {
      fprintf(stderr, "%s%s\n", "Invalid magic number: ", magic);
      fclose(f_serial);
      fclose(f_synth);
      return(0);
   }
   #endif

   /*
    * Loop for each "L_FRAME" speech data
    */
   frame = 0;
   while (fread(serial, sizeof (UWord8), 1, f_serial ) > 0)   // read frame header
   {
      mode = (Word16)((serial[0] >> 3) & 0x0F); // extract mode from header

      fread(&serial[1], sizeof (UWord8), amrnb_frame_size[mode] - 1, f_serial );

      /*--------------------------------------------------------------*
       * Bad frame                                                    *
       *                                                              *
       * bfi = 0 to indicate that we have a correct frame             *
       * bfi = 1 to indicate that we lost the current frame           *
       *--------------------------------------------------------------*/
      bfi = 0;

      D_IF_amrnb_decode( serial, synth, bfi);

      fwrite(synth, sizeof(Word16), L_FRAME8k, f_synth);

      fprintf(stderr, "    Frames processed :  %ld\r", frame++);
   }

   fprintf(stderr, "\n");
   D_IF_amrnb_exit();   // free allocated memory

   fclose(f_serial);
   fclose(f_synth);

   return 0;
}
